@@ -868,71 +868,52 @@ toolbar_build_children (GladeXML *xml, GtkWidget *parent,
 	    if (iconw)
 		gtk_widget_show (iconw);
 
-	    if (new_group) {
-		GtkWidget *toolitem = GTK_WIDGET (gtk_tool_item_new ());
-
-		gtk_container_add (GTK_CONTAINER (parent), toolitem);
-		gtk_widget_show (toolitem);
-	    }
+	    if (new_group)
+		gtk_toolbar_append_space (GTK_TOOLBAR (parent));
 
 	    /* FIXME: these should be translated */
 	    if (!strcmp (childinfo->child->classname, "toggle")) {
-		child = g_object_new (GTK_TYPE_TOGGLE_TOOL_BUTTON,
-				      "label", label,
-				      "stock_id", stock,
-				      NULL);
-		gtk_toggle_tool_button_set_active
-		    (GTK_TOGGLE_TOOL_BUTTON (child), active);
+		child = gtk_toolbar_append_element (
+		    GTK_TOOLBAR (parent),
+		    GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL,
+		    label, tooltip, NULL, iconw, NULL, NULL);
+		gtk_toggle_button_set_active
+		    (GTK_TOGGLE_BUTTON (child), active);
 	    } else if (!strcmp (childinfo->child->classname, "radio")) {
-		child = g_object_new (GTK_TYPE_RADIO_TOOL_BUTTON,
-				      "label", label,
-				      "stock_id", stock,
-				      NULL);
+		child = gtk_toolbar_append_element (
+		    GTK_TOOLBAR (parent),
+		    GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL,
+		    label, tooltip, NULL, iconw, NULL, NULL);
+
 		if (group_name) {
 		    g_object_set (G_OBJECT (child),
 				  "group", glade_xml_get_widget (xml, group_name),
 				  NULL);
 		}
-		gtk_toggle_tool_button_set_active
-		    (GTK_TOGGLE_TOOL_BUTTON (child), active);
-	    } else {
-		child = g_object_new (GTK_TYPE_TOOL_BUTTON,
-				      "label", label,
-				      "stock_id", stock,
-				      NULL);
-	    }
-	    if (iconw)
-		gtk_tool_button_set_icon_widget (GTK_TOOL_BUTTON (child),
-						 iconw);
+		gtk_toggle_button_set_active
+		    (GTK_TOGGLE_BUTTON (child), active);
+	    } else
+		child = gtk_toolbar_append_item (
+		    GTK_TOOLBAR (parent),
+		    label, tooltip, NULL, iconw, NULL, NULL);
 	    
 	    /* GTK+ doesn't support use_underline directly, so we have to hack
 	       it. */
 	    if (use_underline) {
-		GtkWidget *labelw = gtk_tool_button_get_label_widget (GTK_TOOL_BUTTON (child));
-		gtk_label_set_use_underline (GTK_LABEL (labelw), TRUE);
+		GList *elem = g_list_last (GTK_TOOLBAR (parent)->children);
+		GtkToolbarChild *toolbar_child = elem->data;
+		gtk_label_set_use_underline (GTK_LABEL (toolbar_child->label),
+					     TRUE);
 	    }
 
-	    if (tooltip) {
-		gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (child),
-					   xml->priv->tooltips,
-					   tooltip, NULL);
-		
-	    }
-
-	    gtk_container_add (GTK_CONTAINER (parent), child);
-
 	    glade_xml_set_common_params (xml, child, childinfo->child);
 	} else {
 	    child = glade_xml_build_widget (xml, childinfo->child);
 
-	    if (!GTK_IS_TOOL_ITEM (child)) {
-		GtkWidget *toolitem = GTK_WIDGET (gtk_tool_item_new ());
-
-		gtk_container_add (GTK_CONTAINER (toolitem), child);
-		gtk_widget_show (toolitem);
-		child = toolitem;
-	    }
-	    gtk_container_add (GTK_CONTAINER (parent), child);
+	    if (GTK_IS_TOOL_ITEM (child))
+		gtk_toolbar_insert (GTK_TOOLBAR (parent), GTK_TOOL_ITEM (child), -1);
+	    else
+		gtk_toolbar_append_widget (GTK_TOOLBAR (parent), child, NULL, NULL);
 	}
     }
 }
@@ -1236,7 +1217,7 @@ _glade_init_gtk_widgets(void)
     glade_register_widget (GTK_TYPE_FILE_CHOOSER, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_FILE_CHOOSER_DIALOG, glade_standard_build_widget,
-			   NULL, NULL);
+			   gtk_dialog_build_children, dialog_find_internal_child);
     glade_register_widget (GTK_TYPE_FILE_CHOOSER_BUTTON, glade_standard_build_widget,
 			   NULL, NULL);
     glade_register_widget (GTK_TYPE_FILE_SELECTION, NULL,
